package org.artifactory.repo.service.mover;

import org.artifactory.api.common.MoveMultiStatusHolder;
import org.artifactory.repo.LocalRepo;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.RepoRepoPath;
import org.artifactory.repo.StoringRepo;
import org.artifactory.storage.db.fs.model.DbMutableFile;
import org.artifactory.storage.db.fs.session.SqlStorageSession;
import org.artifactory.storage.fs.MutableVfsItem;
import org.artifactory.storage.fs.lock.SessionLockEntry;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import static org.artifactory.repo.service.flexible.tools.MoveCopyTestUtils.prepareCopyOrMoveOverrideMocks;
import static org.artifactory.repo.service.flexible.tools.MoveCopyTestUtils.prepareDbMutableFileMock;
import static org.mockito.Mockito.any;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

public class BaseRepoPathMoverTest extends ArtifactoryHomeBoundTest {

    @Mock
    RepoPath repoPathMock;
    @Mock
    LocalRepo repoMock;
    @Mock
    MutableVfsItem mutableVfsItemMock;
    @Mock
    SqlStorageSession sqlStorageSessionMock;
    @Mock
    SessionLockEntry sessionLockEntryMock;
    @Mock
    MoveMultiStatusHolder moveMultiStatusHolderMock;

    @BeforeMethod
    public void setup() {
        MockitoAnnotations.openMocks(this);
    }

    @Test
    public void testMoveFile() {
        BaseRepoPathMover baseRepoPathMove = mock(BaseRepoPathMover.class, Mockito.CALLS_REAL_METHODS);
        DbMutableFile dbMutableFile = prepareDbMutableFileMock();
        RepoRepoPath<StoringRepo> repoRepoPath =
                prepareCopyOrMoveOverrideMocks(repoPathMock, repoMock, mutableVfsItemMock, sessionLockEntryMock,
                        sqlStorageSessionMock, moveMultiStatusHolderMock, true);

        baseRepoPathMove.moveFile(dbMutableFile, repoRepoPath, moveMultiStatusHolderMock, true);

        verify(repoMock, times(1)).createOrGetFile(any());
        verify(sqlStorageSessionMock, times(1)).save();
        verify(moveMultiStatusHolderMock, times(1)).artifactMoved();
    }
}